#!/usr/bin/env python

import roslib
import sys
import rospy
from cv2 import cv2
import numpy as np
import imutils
from collections import deque
import argparse
from geometry_msgs.msg import Point
from std_msgs.msg import String
from sensor_msgs.msg import Image
from cv_bridge import CvBridge, CvBridgeError

colorBoundLower = (15, 35, 150)
colorBoundHigher = (64, 255, 255)
WIDTH = 500
HEIGHT = 500
emptyImage = np.zeros((HEIGHT,WIDTH,3),np.uint8)


ap = argparse.ArgumentParser()
ap.add_argument("-v", "--video", help="path to the (optional) video file")
ap.add_argument("-b", "--buffer", type=int, default=64, help="max buffer size")
args = vars(ap.parse_args())
pts = deque(maxlen=args["buffer"])


class image_converter:

  def __init__(self):
    self.ballPos_pub = rospy.Publisher("ballPos",Point,queue_size=10)
    self.ballContour_pub = rospy.Publisher("ballContour",Image,queue_size=10)
    self.bridge = CvBridge()
    self.pos = Point()
    
    self.image_sub = rospy.Subscriber("/rawDroneCam",Image,self.callback)

  def findBall(self,frame):
    x = y = radius = 0
    frame = imutils.resize(frame, width=WIDTH, height=HEIGHT)
    blurred = cv2.GaussianBlur(frame, (11, 11), 0)
    hsv = cv2.cvtColor(blurred, cv2.COLOR_BGR2HSV)

    mask = cv2.inRange(hsv, colorBoundLower, colorBoundHigher)
    mask = cv2.erode(mask, None, iterations=4)
    mask = cv2.dilate(mask, None, iterations=2)
    cnts = cv2.findContours(mask.copy(), cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
    cnts = imutils.grab_contours(cnts)
    center = None
    if len(cnts) > 0:
		c = max(cnts, key=cv2.contourArea)
		((x, y), radius) = cv2.minEnclosingCircle(c)
		M = cv2.moments(c)
		center = (int(M["m10"] / (M["m00"]+1e-10)), int(M["m01"] / (M["m00"]+1e-10)))
    pts.appendleft(center)  
    return ((x, y), radius)

  def callback(self,data):
    x = y = radius = 0
    try:
      frame = self.bridge.imgmsg_to_cv2(data, "bgr8")
    except CvBridgeError as e:
      print(e)

    ((x, y), radius) = self.findBall(frame)
    
    if radius > 1:
      emptyImage = np.zeros((HEIGHT,WIDTH,3),np.uint8)
      ballImage = cv2.circle(emptyImage, (int(x), int(y)), int(radius+8),(0, 255, 255), 3)
      ballImage = cv2.circle(ballImage, (int(x), int(y)), 2, (0, 0, 255), 2)
      self.pos.x = x/WIDTH    #normalize 0 is left, 1 is right
      self.pos.y = y/HEIGHT
      self.pos.z = radius
    else:
      self.pos.x = -1
      self.pos.y = -1
      self.pos.z = -1    

    cv2.imshow("Drone Cam", cv2.flip(frame,1))
    cv2.waitKey(3)
    ballImage = self.bridge.cv2_to_imgmsg(ballImage, encoding = 'passthrough')

    try:
      self.ballPos_pub.publish(self.pos)
      self.ballContour_pub.publish(ballImage)
    except CvBridgeError as e:
      print(e)


def main(args):
  rospy.init_node('ball_detector', anonymous=False)
  image_converter()
  try:
    rospy.spin()
  except KeyboardInterrupt:
    print("Shutting down")
  cv2.destroyAllWindows()

if __name__ == '__main__':
    main(sys.argv)
