#!/usr/bin/env python
import roslib
import sys
import rospy
import math
from cv2 import cv2
import numpy as np
import imutils
from collections import deque
import argparse
from geometry_msgs.msg import Point
from std_msgs.msg import String
from sensor_msgs.msg import Image
from cv_bridge import CvBridge, CvBridgeError

WIDTH = 500
HEIGHT = 500
emptyImage = np.zeros((HEIGHT,WIDTH,3),np.uint8)


class line_detector:
    def __init__(self):
        self.lines_pub = rospy.Publisher("linesImage",Image,queue_size=10)
        self.bridge = CvBridge()
        self.image_sub = rospy.Subscriber("/rawDroneCam",Image,self.callback)

    
    def findLines(self,src):
        dst = cv2.Canny(src, 100, 200, None, 3)
    
        # Copy edges to the images that will display the results in BGR
        cdst = cv2.cvtColor(dst, cv2.COLOR_GRAY2BGR)
        cdstP = np.copy(cdst)
    
        emptyImage = np.zeros((HEIGHT,WIDTH,3),np.uint8)
        #lines = cv2.HoughLines(dst, 1, np.pi / 180, 150, None, 0, 0)
        # if lines is not None:
        #     for i in range(0, len(lines)):
        #         rho = lines[i][0][0]
        #         theta = lines[i][0][1]
        #         a = math.cos(theta)
        #         b = math.sin(theta)
        #         x0 = a * rho
        #         y0 = b * rho
        #         pt1 = (int(x0 + 1000*(-b)), int(y0 + 1000*(a)))
        #         pt2 = (int(x0 - 1000*(-b)), int(y0 - 1000*(a)))
        #         lineImage = cv2.line(emptyImage, pt1, pt2, (0,0,255), 3, cv2.LINE_AA)
        
        linesP = cv2.HoughLinesP(dst, 1, np.pi / 180, 100, 20, 100, 20)
    
        if linesP is not None:
            for i in range(0, len(linesP)):
                l = linesP[i][0]
                lineImage = cv2.line(emptyImage, (l[0], l[1]), (l[2], l[3]), (255,0,178), 3, cv2.LINE_AA)
        
        #cv.imshow("Source", src)
        #cv2.imshow("Detected Lines (in red) - Standard Hough Line Transform", cv2.flip(cdst,1))
        #cv2.imshow("Detected Lines - Probabilistic Line Transform", cv2.flip(lineImage,1))
        #cv2.waitKey(3)
        return lineImage


    def callback(self,data):
        try:
            frame = self.bridge.imgmsg_to_cv2(data,"bgr8")
        except CvBridgeError as e:
            print(e)

        linesImage_cv = self.findLines(frame)
        linesImage = self.bridge.cv2_to_imgmsg(linesImage_cv, encoding = 'passthrough')
        try:
            self.lines_pub.publish(linesImage)
        except CvBridgeError as e:
            print(e)


def main(args):
    rospy.init_node('line_detector', anonymous=False)
    line_detector()
    try:
        rospy.spin()
    except KeyboardInterrupt:
        print("shutting down line detector")
    #cv2.destroyAllWindows()
    
if __name__ == "__main__":
    main(sys.argv)