#!/usr/bin/env python

import roslib
import sys
import rospy
from cv2 import cv2
import numpy as np
import imutils
from collections import deque
import argparse
from geometry_msgs.msg import Point
from std_msgs.msg import String
from sensor_msgs.msg import Image
from cv_bridge import CvBridge, CvBridgeError

colorBoundLower = (15, 35, 150)
colorBoundHigher = (64, 255, 255)
WIDTH = 500
HEIGHT = 500
emptyImage = np.zeros((HEIGHT,WIDTH,3),np.uint8)

class image_merger:
  def __init__(self):
    self.ball_contour_sub = rospy.Subscriber("ballContour",Image,self.callbackBall)
    self.lines_sub = rospy.Subscriber("linesImage",Image,self.callbackLines)
    self.bridge = CvBridge()
    self.ballImage = emptyImage

  def callbackBall(self,data):
    try:
      self.ballImage = self.bridge.imgmsg_to_cv2(data, "8UC3")
    except CvBridgeError as e:
      print(e)

    #cv2.imshow("detected ball and lines",self.ballImage)
    #cv2.waitKey(3)

  def callbackLines(self,data):
    try:
      self.lineImage = self.bridge.imgmsg_to_cv2(data, "8UC3")
    except CvBridgeError as e:
      print(e)

    self.combined = self.ballImage + self.lineImage
    cv2.imshow("detected ball and lines",cv2.flip(self.combined,1))
    cv2.waitKey(3)

def main():
  rospy.init_node('merge_cam', anonymous=False)
  image_merger()
  try:
    rospy.spin()
  except KeyboardInterrupt:
    print("Shutting down")
  cv2.destroyAllWindows()

if __name__ == '__main__': 
  main()
